"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const superagent = require("superagent");
const superagentPromise = require("superagent-promise");
const request = superagentPromise(superagent, Promise);
const lruCache = require("lru-cache");
const extendedmind_data_1 = require("./extendedmind-data");
class Utils {
    constructor(backendApiUrl, cacheSettings) {
        this.backendApiUrl = backendApiUrl;
        this.defaultCacheSettings = {
            maxCachedHandles: 100,
            syncTimeTreshold: 60000,
        };
        this.cacheSettings = cacheSettings ? cacheSettings : {};
        for (const defaultSetting in this.defaultCacheSettings) {
            if (this.defaultCacheSettings.hasOwnProperty(defaultSetting) &&
                !this.cacheSettings.hasOwnProperty(defaultSetting)) {
                this.cacheSettings[defaultSetting] = this.defaultCacheSettings[defaultSetting];
            }
        }
    }
    async fetchPublicItems(handle) {
        const publicOwnerUrl = this.backendApiUrl + "/v2/public/" + handle;
        const backendResponse = await request.get(publicOwnerUrl).end();
        if (backendResponse.status === 200) {
            return new extendedmind_data_1.PublicItems(handle, backendResponse.body);
        }
    }
    async refreshPublicItems(handle, cachedItems) {
        if (Date.now() - cachedItems.getLastSynced() > this.cacheSettings.syncTimeTreshold) {
            const publicOwnerModifiedUrl = this.backendApiUrl + "/v2/public/" + handle +
                "?modified=" + cachedItems.getLatestModified();
            const backendResponse = await request.get(publicOwnerModifiedUrl).end();
            if (backendResponse.status === 200) {
                cachedItems.updateItems(backendResponse.body);
            }
        }
        return cachedItems;
    }
    async fetchPublicHeaders() {
        const publicUrl = this.backendApiUrl + "/v2/public";
        const backendResponse = await request.get(publicUrl).end();
        if (backendResponse.status === 200) {
            return new extendedmind_data_1.PublicHeaders(backendResponse.body);
        }
    }
    async refreshPublicHeaders(cachedHeaders) {
        if (Date.now() - cachedHeaders.getLastSynced() > this.cacheSettings.syncTimeTreshold) {
            const publicOwnerModifiedUrl = this.backendApiUrl + "/v2/public" +
                "?modified=" + cachedHeaders.getLatestModified();
            const backendResponse = await request.get(publicOwnerModifiedUrl).end();
            if (backendResponse.status === 200) {
                cachedHeaders.updateHeaders(backendResponse.body);
            }
        }
        return cachedHeaders;
    }
    async getPublicItems(handle) {
        const todayMidnight = new Date().setUTCHours(0, 0, 1, 0);
        if (!this.lastHandleCacheReset || this.lastHandleCacheReset < todayMidnight) {
            this.lastHandleCacheReset = Date.now();
            this.handleCache = lruCache({ max: this.cacheSettings.maxCachedHandles });
        }
        const cachedItems = this.handleCache.get(handle);
        const items = cachedItems ?
            await this.refreshPublicItems(handle, cachedItems) : await this.fetchPublicItems(handle);
        this.handleCache.set(handle, items);
        return items;
    }
    async getPublicHeaders() {
        const todayMidnight = new Date().setUTCHours(0, 0, 1, 0);
        if (!this.lastHeadersReset || this.lastHeadersReset < todayMidnight) {
            this.lastHeadersReset = Date.now();
            this.headers = undefined;
        }
        this.headers = this.headers ?
            await this.refreshPublicHeaders(this.headers) : await this.fetchPublicHeaders();
        return this.headers;
    }
    async getPreviewItem(ownerUUID, itemUUID, previewCode) {
        const backendResponse = await request.get(this.backendApiUrl + "/v2/owners/" + ownerUUID + "/data/items/" +
            itemUUID + "/preview/" + previewCode).end();
        if (backendResponse.status === 200) {
            return extendedmind_data_1.processExternalPublicNote(backendResponse.body);
        }
    }
    async getInfo(latest, history) {
        let url = this.backendApiUrl + "/v2/info";
        if (latest === true) {
            url += "?latest=true";
            if (history === true) {
                url += "&history=true";
            }
        }
        const infoResponse = await request.get(url)
            .set("Accept", "application/json")
            .end();
        return infoResponse.body;
    }
    async getShortId(shortId) {
        if (this.handleCache) {
            const cachedPublicItems = this.handleCache.values();
            for (const cachedPublicItem of cachedPublicItems) {
                const shortIdInfo = cachedPublicItem.getShortId(shortId);
                if (shortIdInfo)
                    return shortIdInfo;
            }
        }
        const backendResponse = await request.get(this.backendApiUrl + "/v2/short/" + shortId).end();
        if (backendResponse.status === 200) {
            return backendResponse.body;
        }
    }
}
exports.Utils = Utils;
//# sourceMappingURL=extendedmind.js.map